# frozen_string_literal: true
require 'horn_of_plenty/models/issue'
require 'horn_of_plenty/adapters/github/client'
require 'horn_of_plenty/adapters/github/responses/list_issues'
require 'horn_of_plenty/adapters/github/requests/list_issues'
require 'chamber'
require 'rspeckled/spec_helpers/rspeckled'

module    HornOfPlenty
module    Adapters
module    Github
module    Responses
describe  ListIssues do
  it 'can retrieve issues' do
    client        = Client.new(url:     'https://api.github.com',
                               version: 'inertia-preview',
                               token:   Chamber.env.adapters.github.token,
                               logger:  $stdout)
    issue_request = Requests::ListIssues.new(
                      repository: 'thekompanee/hustle-and-flow-issue-dump',
    )

    issue_response = client.request(issue_request)

    expect(issue_response.status_code).to eql 200
    expect(issue_response.body.length).to eql 3
    expect(issue_response).to             be_successful
    expect(issue_response.body).to        include(
      a_hash_including(
        'title' => 'My Bar Pull Request',
      ),
    )
  end

  it 'can filter issues' do
    client        = Client.new(url:     'https://api.github.com',
                               version: 'inertia-preview',
                               token:   Chamber.env.adapters.github.token,
                               logger:  $stdout)
    issue_request = Requests::ListIssues.new(
                      repository: 'thekompanee/hustle-and-flow-issue-dump',
                      query:      {
                        author_id: 'jfelchner',
                        status:    'open',
                        labels:    'dont-touch-this-label',
                      },
    )

    issue_response = client.request(issue_request)

    expect(issue_response.status_code).to eql 200
    expect(issue_response.body.length).to eql 1
    expect(issue_response).to             be_successful
    expect(issue_response.body).to        include(
      a_hash_including(
        'title' => 'Menu item specifications not working',
      ),
    )
  end

  it 'can handle if a issue does not exist' do
    client        = Client.new(url:     'https://api.github.com',
                               version: 'inertia-preview',
                               token:   Chamber.env.adapters.github.token,
                               logger:  $stdout)
    issue_request = Requests::ListIssues.new(repository: 'thekompanee/hustle_and_flow')

    issue_response = client.request(issue_request)

    expect(issue_response.status_code).to eql 200
    expect(issue_response.body).to        be_empty
    expect(issue_response).to             be_successful
  end

  it 'can handle if there is an error' do
    client        = Client.new(url:     'https://api.github.com',
                               version: 'inertia-preview',
                               token:   Chamber.env.adapters.github.token,
                               logger:  $stdout)
    issue_request = Requests::ListIssues.new(repository: 'thekompanee/no-repo-here')

    expect { client.request(issue_request) }.to raise_error Github::Errors::NotFound
  end

  it 'can parse the issues that are returned' do
    client        = Client.new(url:     'https://api.github.com',
                               version: 'inertia-preview',
                               token:   Chamber.env.adapters.github.token,
                               logger:  $stdout)
    issue_request = Requests::ListIssues.new(
                      repository: 'thekompanee/hustle-and-flow-issue-dump',
    )

    issue_response = client.request(issue_request)
    issue          = issue_response.result.first

    expect(issue).to       be_a HornOfPlenty::Models::Issue
    expect(issue.title).to eql 'My Bar Pull Request'
  end
end
end
end
end
end
