# frozen_string_literal: true
require 'horn_of_plenty/models/pull_request'
require 'horn_of_plenty/adapters/github/client'
require 'horn_of_plenty/adapters/github/responses/list_pull_requests'
require 'horn_of_plenty/adapters/github/requests/list_pull_requests'
require 'chamber'
require 'rspeckled/spec_helpers/rspeckled'

module    HornOfPlenty
module    Adapters
module    Github
module    Responses
describe  ListPullRequests do
  it 'can retrieve pull requests' do
    client               = Client.new(url:     'https://api.github.com',
                                      version: 'inertia-preview',
                                      token:   Chamber.env.adapters.github.token,
                                      logger:  $stdout)
    pull_request_request = Requests::ListPullRequests.new(
      repository: 'thekompanee/hustle-and-flow-issue-dump',
    )

    pull_request_response = client.request(pull_request_request)

    expect(pull_request_response.status_code).to eql 200
    expect(pull_request_response.body.length).to eql 1
    expect(pull_request_response).to             be_successful
    expect(pull_request_response.body).to        include(
      a_hash_including(
        'title' => 'My Bar Pull Request',
      ),
    )
  end

  it 'can filter pull requests' do
    client               = Client.new(url:     'https://api.github.com',
                                      version: 'inertia-preview',
                                      token:   Chamber.env.adapters.github.token,
                                      logger:  $stdout)
    pull_request_request = Requests::ListPullRequests.new(
      repository: 'thekompanee/hustle-and-flow-issue-dump',
      query:      {
        author_id: 'jfelchner',
        status:    'open',
        labels:    'dont-touch-this-label',
      },
    )

    pull_request_response = client.request(pull_request_request)

    expect(pull_request_response.status_code).to eql 200
    expect(pull_request_response.body.length).to eql 1
    expect(pull_request_response).to             be_successful
    expect(pull_request_response.body).to        include(
      a_hash_including(
        'title' => 'My Bar Pull Request',
      ),
    )
  end

  it 'can handle if a pull request does not exist' do
    client               = Client.new(url:     'https://api.github.com',
                                      version: 'inertia-preview',
                                      token:   Chamber.env.adapters.github.token,
                                      logger:  $stdout)
    pull_request_request = Requests::ListPullRequests.new(
      repository: 'thekompanee/hustle_and_flow',
    )

    pull_request_response = client.request(pull_request_request)

    expect(pull_request_response.status_code).to eql 200
    expect(pull_request_response.body).to        be_empty
    expect(pull_request_response).to             be_successful
  end

  it 'can handle if there is an error' do
    client               = Client.new(url:     'https://api.github.com',
                                      version: 'inertia-preview',
                                      token:   Chamber.env.adapters.github.token,
                                      logger:  $stdout)
    pull_request_request = Requests::ListPullRequests.new(
      repository: 'thekompanee/no-repo-here',
    )

    expect { client.request(pull_request_request) }.to raise_error \
      Github::Errors::NotFound
  end

  it 'can parse the pull requests that are returned' do
    client               = Client.new(url:     'https://api.github.com',
                                      version: 'inertia-preview',
                                      token:   Chamber.env.adapters.github.token,
                                      logger:  $stdout)
    pull_request_request = Requests::ListPullRequests.new(
      repository: 'thekompanee/hustle-and-flow-issue-dump',
    )

    pull_request_response = client.request(pull_request_request)
    pull_request          = pull_request_response.result.first

    expect(pull_request).to       be_a HornOfPlenty::Models::PullRequest
    expect(pull_request.title).to eql 'My Bar Pull Request'
  end
end
end
end
end
end
